"""
    ！！！封装成类会导致「segment fault」，原因未知。
"""

import fastdeploy as fd
import cv2
import os
import time

import PMP.src_dsr.src_config as src_config


class OCRAlg:
    def __init__(self, use_cls=False, device_id=0, cls_bs=1, rec_bs=1) -> None:
        super(OCRAlg, self).__init__()
        self.ppocr_v3 = self.init_ocr(device_id, cls_bs, rec_bs)
    
    def init_ocr(self,use_cls=False, device_id=0, cls_bs=1, rec_bs=1):
        det_option = fd.RuntimeOption()
        cls_option = fd.RuntimeOption()
        rec_option = fd.RuntimeOption()

        det_option.use_gpu(device_id)
        cls_option.use_gpu(device_id)
        rec_option.use_gpu(device_id)

        det_option.use_trt_backend()
        cls_option.use_trt_backend()
        rec_option.use_trt_backend()

         # If use TRT backend, the dynamic shape will be set as follow.
        # We recommend that users set the length and height of the detection model to a multiple of 32.
        # We also recommend that users set the Trt input shape as follow.
        det_option.set_trt_input_shape("x", [1, 3, 64, 64], [1, 3, 640, 640],
                                       [1, 3, 960, 960])
        cls_option.set_trt_input_shape("x", [1, 3, 48, 10],
                                       [cls_bs, 3, 48, 320],
                                       [cls_bs, 3, 48, 1024])
        rec_option.set_trt_input_shape("x", [1, 3, 48, 10],
                                       [rec_bs, 3, 48, 320],
                                       [rec_bs, 3, 48, 2304])

        # Users could save TRT cache file to disk as follow.
        det_option.set_trt_cache_file(src_config.model_ocr_det_path + "/det_trt_cache.trt")
        cls_option.set_trt_cache_file(src_config.model_ocr_cls_path + "/cls_trt_cache.trt")
        rec_option.set_trt_cache_file(src_config.model_ocr_rec_path + "/rec_trt_cache.trt")


        det_model_file = os.path.join(src_config.model_ocr_det_path, "inference.pdmodel")
        det_params_file = os.path.join(src_config.model_ocr_det_path, "inference.pdiparams")

        print(f"det_model_file: {det_model_file}")

        cls_model_file = os.path.join(src_config.model_ocr_cls_path, "inference.pdmodel")
        cls_params_file = os.path.join(src_config.model_ocr_cls_path, "inference.pdiparams")

        rec_model_file = os.path.join(src_config.model_ocr_rec_path, "inference.pdmodel")
        rec_params_file = os.path.join(src_config.model_ocr_rec_path, "inference.pdiparams")
        rec_label_file = src_config.ocr_keys_path

        det_model = fd.vision.ocr.DBDetector(
            det_model_file, det_params_file, runtime_option=det_option)

        cls_model = fd.vision.ocr.Classifier(
            cls_model_file, cls_params_file, runtime_option=cls_option)

        rec_model = fd.vision.ocr.Recognizer(
            rec_model_file, rec_params_file, rec_label_file, runtime_option=rec_option)
        

        # Parameters settings for pre and post processing of Det/Cls/Rec Models.
        # All parameters are set to default values.
        det_model.preprocessor.max_side_len = 960
        det_model.postprocessor.det_db_thresh = 0.3
        det_model.postprocessor.det_db_box_thresh = 0.6
        det_model.postprocessor.det_db_unclip_ratio = 1.5
        det_model.postprocessor.det_db_score_mode = "slow"
        det_model.postprocessor.use_dilation = False
        cls_model.postprocessor.cls_thresh = 0.9

        # Create PP-OCRv3, if cls_model is not needed, just set cls_model=None .
        cls_model = cls_model if use_cls is True else None

        ppocr_v3 = fd.vision.ocr.PPOCRv3(
            det_model=det_model, cls_model=cls_model, rec_model=rec_model)

        # Set inference batch size for cls model and rec model, the value could be -1 and 1 to positive infinity.
        # When inference batch size is set to -1, it means that the inference batch size
        # of the cls and rec models will be the same as the number of boxes detected by the det model.
        ppocr_v3.cls_batch_size = cls_bs
        ppocr_v3.rec_batch_size = rec_bs

        return ppocr_v3
    
    def predict(self, bgr_img):
        start = time.perf_counter()
        result = self.ppocr_v3.predict(bgr_img)
        vis_im = fd.vision.vis_ppocr(bgr_img, result)
        print(f"OCR cost {(time.perf_counter() - start)*1000 :.2f} ms")

        return result, vis_im