#!/usr/bin/env python
# -*- coding: UTF-8 -*-
'''
@Project ：Filter_Object 
@File ：fd_yolo.py
@IDE ：PyCharm 
@Author ：zjj
@Date ：2024/1/19 10:45 
'''
import fastdeploy as fd
from loguru import logger

from config import POLICE_PATH


class FdYolov8(object):
    def __init__(self, model_path=POLICE_PATH, device="gpu", use_trt=False, half=False):
        self.model_path = model_path
        self.device = device
        self.use_trt = use_trt
        self.half = half
        self.model = None
        # 加载模型
        self.init_model()

    def build_option(self):
        """
        构建运行时选项
        :return: option
        """
        option = fd.RuntimeOption()
        if self.device.lower() == "gpu":
            option.use_gpu()
        if self.use_trt:
            option.use_trt_backend()
            if self.half:
                option.trt_option.enable_fp16 = True
        return option

    def init_model(self):
        runtime_option = self.build_option()
        self.model = fd.vision.detection.YOLOv8(
            self.model_path, runtime_option=runtime_option
        )
        logger.info('fastdeploy yolo模型加载成功！')

    def predict(self, bgr_img):
        result = self.model.predict(bgr_img)
        return result