#!/usr/bin/env python
# -*- coding: UTF-8 -*-
'''
@Project ：Filter_Object 
@File ：main_v2_test.py
@IDE ：PyCharm 
@Author ：zjj
@Date ：2024/1/19 9:51 
'''
import os
import time
import unittest
import uuid
from pathlib import Path
import cv2
from main_v2 import TrackMain
from utils import extract_yolo_results

file_name = 'AM03778_09060120200408073818_0015A.MP4'


def make_result_dir():
    # 保存图片，可视化
    # dir_name = str(uuid.uuid4())[:7]
    dir_name = file_name
    dirx = Path(r'/mnt/large/zhoujinjuan_data/data/result') / dir_name
    if not dirx.exists():
        dirx.mkdir()
        print(f'tmp dir {dirx}')
    return dirx


class TestMain(unittest.TestCase):
    def setUp(self):
        self.use_trt = True
        self.half = False
        self.obj = TrackMain(device='gpu', use_trt=self.use_trt, half=self.half)
        video_path = os.path.join('/mnt/large/zhoujinjuan_data/data/tp_videos/20240122-test/' + file_name)
        self.cap = cv2.VideoCapture(video_path)
        # 记录视频所有满足条件的人脸
        self.all_face_embeds = []
        # 记录视频所有满足条件的车牌号
        self.all_licenses_list = []
        self.frame_count = 0
        # 跳帧：每x帧处理一帧
        self.skip_frame = 1
        print('init success')

    def process_one_frame_v2(self, frame):
        results = self.obj.yolo_seg_model(frame)
        # 提取模型检测结果 infos=[xywhs, cls, scores, xy_list]
        is_hit, infos = extract_yolo_results(results)
        if not is_hit:
            print('no target detect')
            return [], [], frame

        # yolo检测结果可视化
        # annotated_frame = results[0].plot()
        # cv2.imwrite('data/yolo_result.png', annotated_frame)

        # 阈值筛选
        tinfos_each_frame = self.obj.statistics_one_frame(infos)
        # print(tinfos_each_frame)

        # 保持原接口不动，比process_one_frame 多返回一个frame_copy
        face_embeddings, licenses, frame_copy = self.obj.process_topn_in_one_frame(frame, tinfos_each_frame)
        return face_embeddings, licenses, frame_copy

    def test_process_one_frame(self):
        """
        测试一帧的检测结果
        :return:
        """
        for index, name in enumerate(['truck2.png']):
            pathx = 'data/' + name
            frame = cv2.imread(pathx)
            _, licenses, frame_copy = self.process_one_frame_v2(frame)
            print(f'licenses {licenses}')
            cv2.imwrite('data/result_' + str(index) + '.png', frame_copy)

    def test_main(self):
        print('begin test')
        # 先创建结果目录
        result_dir = make_result_dir()
        start = time.time()
        while self.cap.isOpened():
            ret, frame = self.cap.read()
            if not ret:
                break
            self.frame_count += 1

            # todo 考虑跳帧
            if self.frame_count % self.skip_frame != 0:
                continue

            # 每一帧的处理结果
            face_embeds, license_list, frame_copy = self.process_one_frame_v2(frame)

            # todo test 测试过程中保存图片可视化
            if face_embeds and not license_list:
                name = str(uuid.uuid4())[:7]
                cv2.imwrite((result_dir / ('face_' + name + '.png')).as_posix(), frame_copy)
            elif not face_embeds and license_list:
                name = str(uuid.uuid4())[:7]
                cv2.imwrite((result_dir / ('motorVehicle_' + name + '.png')).as_posix(), frame_copy)
            elif face_embeds and license_list:
                name = str(uuid.uuid4())[:7]
                cv2.imwrite((result_dir / ('all_' + name + '.png')).as_posix(), frame_copy)

            self.all_face_embeds.extend(face_embeds)
            self.all_licenses_list.extend(license_list)
        end = time.time()
        t = round(end - start, 2)
        actual_frame_number = round(self.frame_count / self.skip_frame)
        print(f'frame count {self.frame_count}\nactual deal frame count {actual_frame_number}\ntime {t}s'
              f'\nface embeddings count {len(self.all_face_embeds)}'
              f'\nlicenses count {len(self.all_licenses_list)}')
        print(self.all_licenses_list)
        print(f'use_trt={self.use_trt}\nhalf={self.half}\n')


if __name__ == '__main__':
    unittest.main()