#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@Project ：yolo_demos 
@File ：utils.py
@IDE ：PyCharm 
@Author ：zjj
@Date ：2023/9/21 10:15 
"""
import os.path
import re
import shutil

from algs import ocr_alg
from pathlib import Path
import cv2
import numpy as np

# from track.main import TrackMain
from config import (
    BIG_FRAME_NUM,
    LABEL_NAMES,
    FACE_SAVE_THRESHOLD,
    DIR_RATIO_THRESHOLD,
    PERSON,
    CAR,
    FACE_COMPARE_THRESHOLD,
    ARTICULATION_MODEL_PATH,
    ARTICULATION_RANGE_PATH,
    POLICE_IOU, DELTA_Y,
)


def save_img(track_id, label, frame, save_dir, origin=None):
    dir_name = str(track_id) + "_" + label
    dirx = save_dir / dir_name
    if not dirx.exists():
        dirx.mkdir()
    fnames = os.listdir(dirx.as_posix())
    lens = len(fnames)
    name = str(lens)
    # name = str(uuid.uuid4())[:6]
    out_path = dirx / f"{track_id}_{name}.png"
    cv2.imwrite(out_path.as_posix(), frame)
    if origin is not None:
        out_path = dirx / f"{track_id}_{name}_origin.png"
        cv2.imwrite(out_path.as_posix(), origin)


def save_car(license_plate, img, save_dir):
    """
    按车牌号分目录保存车图片
    :param license_plate:
    :param img:
    :param save_dir:
    :return:
    """
    dir_name = license_plate + "_" + CAR
    dirx = save_dir / dir_name
    if not dirx.exists():
        dirx.mkdir()
    fnames = os.listdir(dirx.as_posix())
    lens = len(fnames)
    name = str(lens)
    out_path = dirx / f"{name}.png"
    cv2.imwrite(out_path.as_posix(), img)


def filter_by_big_box(big_box_count):
    """
    对每个类别下每个目标的大框图数量，和阈值比较，保留满足阈值的tid
    :param big_box_count:
    :return:
    """
    result = dict()
    for label, tid_count in big_box_count.items():
        result[label] = []
        for tid, count in tid_count.items():
            if count > BIG_FRAME_NUM[label]:
                result[label].append(tid)
    return result


def get_face_tids(save_dir):
    """
    过滤没有保存人脸的target id
    :return:
    """
    face_tids = []
    for pathx in save_dir.iterdir():
        stem = pathx.stem
        tid, label = stem.split("_")
        if label == PERSON:
            face_tids.append(int(tid))
    return face_tids


# print(get_face_tids())
# def get_():
#     import cv2
#     from ultralytics import YOLO
#     import numpy as np
#     import torch
#
#     img = cv2.imread('ultralytics/assets/bus.jpg')
#     model = YOLO('yolov8m-seg.pt')
#     results = model.predict(source=img.copy(), save=True, save_txt=False, stream=True)
#     for result in results:
#         # get array results
#         masks = result.masks.masks
#         boxes = result.boxes.boxes
#         # extract classes
#         clss = boxes[:, 5]
#         # get indices of results where class is 0 (people in COCO)
#         people_indices = torch.where(clss == 0)
#         # use these indices to extract the relevant masks
#         people_masks = masks[people_indices]
#         # scale for visualizing results
#         people_mask = torch.any(people_masks, dim=0).int() * 255
#         # save to file
#         cv2.imwrite(str(model.predictor.save_dir / 'merged_segs.jpg'), people_mask.cpu())


def get_target(frame, polygon, p1, p2):
    # 转灰度
    gray_img = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
    mask = np.zeros_like(gray_img)
    # 轮廓内白-255， 外黑-0
    cv2.polylines(mask, [polygon], True, (255, 0, 0), thickness=1)
    cv2.fillPoly(mask, pts=[polygon], color=(255, 255, 255, 128), lineType=cv2.LINE_AA)
    # cv2.imwrite('../data/track/polygon.png', mask)

    # mask 人-255， 背景-0
    # 转成 人-1 背景-0
    _, binary_image = cv2.threshold(mask, 128, 1, cv2.THRESH_BINARY)
    # 背景黑色，人像
    frame[:, :, 0] = frame[:, :, 0] * binary_image
    frame[:, :, 1] = frame[:, :, 1] * binary_image
    frame[:, :, 2] = frame[:, :, 2] * binary_image
    # cv2.imwrite('../data/track/person.png', frame)

    # 转成 人-0 背景-255
    _, white_bg = cv2.threshold(mask, 128, 255, cv2.THRESH_BINARY_INV)
    frame[:, :, 0] = frame[:, :, 0] + white_bg
    frame[:, :, 1] = frame[:, :, 1] + white_bg
    frame[:, :, 2] = frame[:, :, 2] + white_bg
    # cv2.imwrite('../data/track/new.png', frame)

    # xyxy = results[0].boxes.xyxy[0].cpu().numpy().astype(int)
    target = frame[p1[1] : p2[1], p1[0] : p2[0]]
    # cv2.imwrite('../data/track/12.png', person)
    return target


def get_target_mask(frame, polygon):
    # 转灰度
    frame_copy = frame.copy()
    gray_img = cv2.cvtColor(frame_copy, cv2.COLOR_BGR2GRAY)
    mask = np.zeros_like(gray_img)
    # 轮廓内白-255， 外黑-0
    cv2.polylines(mask, [polygon], True, (255, 0, 0), thickness=1)
    cv2.fillPoly(mask, pts=[polygon], color=(255, 255, 255, 128), lineType=cv2.LINE_AA)
    # cv2.imwrite('../data/track/polygon.png', mask)

    # mask 人-255， 背景-0
    # 转成 人-1 背景-0
    _, binary_image = cv2.threshold(mask, 128, 1, cv2.THRESH_BINARY)
    # 背景黑色，人像
    # frame_copy[:, :, 0] = frame_copy[:, :, 0] * binary_image
    # frame_copy[:, :, 1] = frame_copy[:, :, 1] * binary_image
    # frame_copy[:, :, 2] = frame_copy[:, :, 2] * binary_image
    # cv2.imwrite('../data/track/person.png', frame_copy)
    return binary_image


def get_coordinate(xywh):
    # xy是框的中心点坐标 wh是框的宽和高
    x, y, w, h = xywh
    xmin, ymin = int(x - w / 2), int(y - h / 2)
    xmax, ymax = int(x + w / 2), int(y + h / 2)
    p1 = (xmin, ymin)
    p2 = (xmax, ymax)
    return p1, p2


def show_mid_loc_statistics(label_tids, left_tids, mid_loc_count):
    """
    统计每个类别下有哪些目标id，以及各个目标id位于图片中间位置的次数
    :param label_tids: key：类别id；value：该类别下目标id列表
    :param left_tids: 筛选后留下的目标id列表
    :param mid_loc_count: key：目标id；value：int，该目标位于图像中间位置的次数
    :return:
    """
    # 获取每个类别剩余候选对象数目，同时每个目标位于中间位置的帧数
    left_label_tids = dict()
    for key, value in LABEL_NAMES.items():
        left_label_tids[key] = []
    for tid in left_tids:
        for cid, tid_list in label_tids.items():
            if tid in tid_list:
                left_label_tids[cid].append(tid)
    for key, tid_list in left_label_tids.items():
        # 位居中间位置的数目
        mid_statistics = [
            (tid, count) for tid, count in mid_loc_count.items() if tid in tid_list
        ]
        mid_statistics.sort(key=lambda x: x[1], reverse=True)
        print(f"{LABEL_NAMES[key]}: \n\t{tid_list}\n\t{mid_statistics}")
    return left_label_tids


# def docarray_sim(left_label_tids, matcher):
#     """
#     对 left_label_tids 中每个类别下的目标列表，计算同类别的不同目标文件夹之间的相似，
#     :param left_label_tids:
#     :return:
#     """
#     pairs = dict()
#     for cid, tid_list in left_label_tids.items():
#         checked = []
#         for i in range(len(tid_list)):
#             if tid_list[i] == 265:
#                 print()
#             pairs[tid_list[i]] = []
#             if i in checked:
#                 continue
#             dir_i = str(tid_list[i]) + '_' + LABEL_NAMES[cid]
#             for j in range(i + 1, len(tid_list)):
#                 if tid_list[j] == 3483:
#                     print('')
#                 if j in checked:
#                     continue
#                 dir_j = str(tid_list[j]) + '_' + LABEL_NAMES[cid]
#                 # 目标i文件夹 相对 目标j文件夹，i文件夹中的图片有40%比例在文件夹j中都能找到相似的，即认为两个文件夹相似
#                 ratio = matcher.get_sim_ratio(left_path=SAVE_DIR / dir_i, right_path=SAVE_DIR / dir_j)
#                 if ratio > 0.4:
#                     checked.append(j)
#                     pairs[tid_list[i]].append(tid_list[j])
#     return pairs


# data_dir = '../data/track/target_images1'
# matcher = ImageMatcher()
# x = {0: [265, 2955, 1676, 2197, 3483, 160, 551, 1589, 2870, 2363, 1478, 2759, 2504, 586, 3532, 3157, 1879, 2015, 3296, 2149, 1766, 745, 1773, 878, 1902, 3056, 1014, 252, 2814]}
#
# t1 = time.time()
# print(docarray_sim(x, matcher))
# t2 = time.time()
# print(round(t2 - t1, 2))


def cosine_similarity(a, b):
    """
    计算余弦相似度
    :param a:
    :param b:
    :return:
    """
    a = np.array(a)
    b = np.array(b)
    # L2范数
    mul_a = np.linalg.norm(a, ord=2)
    mul_b = np.linalg.norm(b, ord=2)
    mul_ab = np.dot(a, b)
    return mul_ab / (mul_a * mul_b)


def filter_and_save_face_old(
    face, track_id, last_face, last_track_id, face_det, save_dir, annotated_frame=None
):
    """
    当前人脸目标id和上一个不同，则直接保存当前人脸；
    和上一个相同，则计算两次人脸相似，相似的则不保存当前；
    :param face:
    :param track_id:
    :param last_face:
    :param last_track_id:
    :param face_det: 人脸比对模型
    :param save_dir:
    :param annotated_frame: 目标检测可视化结果，可保存用于调试
    :return:
    """
    # 目标id不同，直接保存
    if track_id != last_track_id:
        # save_img(track_id, PERSON, face, save_dir, origin=annotated_frame)
        save_img(track_id, PERSON, face, save_dir)
        return face, track_id

    # 副本进去embedding！！！
    cos = face_det.face_compare(face, last_face)
    if cos >= FACE_SAVE_THRESHOLD:
        return last_face, last_track_id
    else:
        # debug
        # save_img(track_id, PERSON, face, save_dir, origin=annotated_frame)
        save_img(track_id, PERSON, face, save_dir)
        return face, track_id


def filter_and_save_face(face, track_id, face_det, save_dir, annotated_frame=None):
    """
    当前人脸目标id和上一个不同，则直接保存当前人脸；
    和上一个相同，则计算两次人脸相似，相似的则不保存当前；
    :param face:
    :param track_id:
    :param face_det: 人脸比对模型
    :param save_dir:
    :param annotated_frame: 目标检测可视化结果，可保存用于调试
    :return:
    """
    dir_name = str(track_id) + "_" + PERSON
    dirx = save_dir / dir_name
    if not dirx.exists():
        dirx.mkdir()
        save_img(track_id, PERSON, face, save_dir)
        return

    # 在所属文件夹内，找最后一次保存的文件，和当前文件比较
    file_stems = [(int(filex.stem.split("_")[1]), filex) for filex in dirx.iterdir()]
    file_stems.sort(key=lambda x: x[0])

    last_face = cv2.imread(file_stems[-1][-1].as_posix())

    # 副本进去embedding！！！
    cos = face_det.face_compare(face, last_face)
    if cos >= FACE_SAVE_THRESHOLD:
        return
    else:
        # debug
        # save_img(track_id, PERSON, face, save_dir, origin=annotated_frame)
        save_img(track_id, PERSON, face, save_dir)
        return


def get_color(idx):
    idx = idx * 3
    color = ((37 * idx) % 255, (17 * idx) % 255, (29 * idx) % 255)
    return color


def draw_rectangle_text(annotated_frame, track_id, p1, p2, label, conf, num, s):
    img_h, img_w, _ = annotated_frame.shape
    annotated_frame = cv2.rectangle(annotated_frame, p1, p2, get_color(track_id), 5)
    text = f"id:{track_id} {label} {conf} n:{num} s:{s}"

    # 文字左下角坐标
    org = p1
    font = cv2.FONT_HERSHEY_SIMPLEX
    font_scale = 1
    thickness = 3
    # (width, height)
    text_size, _ = cv2.getTextSize(text, font, font_scale, thickness)
    # 防止文本超出图片范围
    if text_size[0] + org[0] > img_w:
        org = (img_w - text_size[0], org[1])
    if text_size[1] > org[1]:
        org = (org[0], text_size[1])

    annotated_frame = cv2.putText(
        annotated_frame,
        text,
        org,
        font,
        font_scale,
        color=get_color(track_id),
        thickness=thickness,
        lineType=cv2.LINE_AA,
    )
    return annotated_frame


def filter_sim_tid(face_det, tid_dirs, save_dir):
    """
    对多个不同track_id文件夹，计算两两相似，合并同目标，减少目标候选
    :param face_det:
    :param tid_dirs:
    :return:
    """
    checked = []
    dir_lens = len(tid_dirs)
    # 相似目标聚类
    target_clusters = dict()
    for i in range(dir_lens):
        if i in checked:
            continue
        max_info = [0, 0]
        target_clusters[tid_dirs[i]] = {"clusters": [], "max_info": tuple()}
        dir_i = save_dir / (str(tid_dirs[i]) + "_" + PERSON)

        for j in range(i + 1, dir_lens):
            if j in checked:
                continue
            dir_j = save_dir / (str(tid_dirs[j]) + "_" + PERSON)
            ratio_i, ratio_j, max_value_index = compare_dir_i_j(dir_i, dir_j, face_det)
            # print(f'{dir_i.name} {dir_j.name} {ratio_i} {ratio_j}')
            if ratio_i > DIR_RATIO_THRESHOLD and ratio_j > DIR_RATIO_THRESHOLD:
                checked.append(j)
                # 记录相似目标
                target_clusters[tid_dirs[i]]["clusters"].append(tid_dirs[j])
                if max_value_index[0] > max_info[0]:
                    max_info = max_value_index
        # 记录文件夹i中相似度最大的原文件索引和最大相似度值
        target_clusters[tid_dirs[i]]["max_info"] = max_info
    return target_clusters


def compare_dir_i_j(dir_i, dir_j, face_det):
    """
    比较两个文件夹中的图片，i文件夹中每个图片，和j文件夹中每个图片计算相似，找到满足相似条件的则计数+1，然后继续比较i文件夹下一个图片
    :param dir_i:
    :param dir_j:
    :param face_det:
    :return:
    ratio_i: 文件夹i中在文件夹j中有满足相似条件的文件数 占 文件夹i中文件总数的占比
    ratio_j: 文件夹j中在文件夹i中有满足相似条件的文件数 占 文件夹j中文件总数的占比
    max_value: 两个文件夹的embedding矩阵计算cos相似，最大的相似度值；
    max_2d_index：最大值在相似矩阵中的二维坐标，只取行坐标，即文件夹i中的文件索引
    """
    if not dir_i.exists() or not dir_j.exists():
        return 0, 0, tuple()

    # 加上i文件夹m个文件，j文件夹n个文件
    matrix_i, stems_i = get_dir_embed_matrix(dir_i, face_det)
    matrix_j, stems_j = get_dir_embed_matrix(dir_j, face_det)
    # m*n
    result = matrix_i.dot(matrix_j.T)
    sim_result = result >= FACE_COMPARE_THRESHOLD
    sim_result = sim_result.astype(np.float32)

    # i中每个文件在j中有多少个满足相似条件的 (m, 1)
    sim_num = np.sum(sim_result, axis=-1, keepdims=True)
    # 只要有一个满足即可计数+1 todo 如何文件夹a中所有文件都和b中某一个文件相似怎么办？
    count = (sim_num > 0).astype(np.float32)
    count = np.sum(count, axis=0)
    ratio_i = round(count[0] / sim_num.shape[0], 2)

    # (1, n) 文件夹j中有多少个匹配文件夹i的，防止文件夹j中有多个文件，只有一个文件匹配文件夹i中的所有文件的情况
    sim_num = np.sum(sim_result, axis=0, keepdims=True)
    count = (sim_num > 0).astype(np.float32)
    count = np.sum(count, axis=-1)
    ratio_j = round(count[0] / sim_num.shape[1], 2)

    # 获取最大值的索引
    max_2d_index = np.unravel_index(np.argmax(result), result.shape)
    max_value = round(result[max_2d_index], 2)

    return ratio_i, ratio_j, (max_value, max_2d_index[0])


def get_dir_embed_matrix(dirx, face_det):
    """
    将文件夹下面所有文件embedding，放一个矩阵中
    :param dirx:
    :param face_det:
    :return:
    """
    # 每个文件embedding结果列表
    results = []
    # 对应embedding顺序的每个文件名的索引部分，比如 140_3.png 取3
    stems = []
    for pathx in dirx.iterdir():
        img = cv2.imread(pathx.as_posix())
        embed = face_det.get_img_embedding(img)
        results.append(embed)
        stems.append(pathx.stem.split("_")[1])

    result_matrix = np.stack(results)
    # todo 归一化 否则矩阵乘积结果可能大于1
    result_matrix = result_matrix / np.linalg.norm(
        result_matrix, axis=-1, keepdims=True
    )
    return result_matrix, stems


def extract_yolo_results(results):
    # Get the boxes and track IDs
    boxes = results[0].boxes
    masks = results[0].masks
    # is_track 为False时，目标id是None，即没有追踪
    # if not masks or not boxes.is_track:
    if not masks:
        return False, None

    # 轮廓坐标
    xy_list = masks.xy
    # xy是框的中心点坐标 wh是框的宽和高
    xywhs = boxes.xywh.cpu().tolist()

    # 类标list
    cls = boxes.cls.cpu().tolist()
    # 置信度list
    scores = boxes.conf.cpu().tolist()
    return True, [xywhs, cls, scores, xy_list]


def predict_ocr(frame):
    # OCR检测
    license_plate_list = []
    ocr_result = ocr_alg.ocr_predict(frame)
    # 常规车牌号识别
    for txt in ocr_result[0].text:
        # 车牌
        if len(txt) in [7, 8, 9]:
            plate_num = parse_plate_number(txt)
            if plate_num is not None:
                # 中间漏点的补上，通常是大卡车车厢上的车牌被识别
                if len(txt) == 7:
                    plate_num = plate_num[:2] + '·' + plate_num[2:]
                license_plate_list.append(plate_num)

    # todo 20240124 摩托车车牌规则匹配，通常是上下两行文本(不管上面是否检测到车牌)
    moto_list = parse_moto_number(ocr_result)

    license_plate_list.extend(moto_list)
    return license_plate_list


# 使用正则解析车牌号
def parse_plate_number(txt):
    pattern = "^[京津沪渝冀豫云辽黑湘皖鲁新苏浙赣鄂桂甘晋蒙陕吉闽贵粤青藏川宁琼使领A-Z]{1}[A-Z]{1}[·]?[A-Z0-9]{4}[A-Z0-9挂学警港澳]{1}$"
    pattern1 = "^[京津沪渝冀豫云辽黑湘皖鲁新苏浙赣鄂桂甘晋蒙陕吉闽贵粤青藏川宁琼使领A-Z]{1}[A-Z]{1}[·]?[A-Z0-9]{5}[A-Z0-9挂学警港澳]{1}$"
    match = re.match(pattern, txt)
    match1 = re.match(pattern1, txt)
    if match:
        return match.group()
    if match1:
        return match1.group()


def parse_moto_number(ocr_result):
    """
    摩托车车牌规则匹配，通常是上下两行文本
    :param ocr_result:
    :return:
    """
    license_plate_list = []
    p_before = "^[京津沪渝冀豫云辽黑湘皖鲁新苏浙赣鄂桂甘晋蒙陕吉闽贵粤青藏川宁琼使领A-Z]{1}[·]?[A-Z]{1}$"
    p_end = "^[A-Z0-9]{4}[A-Z0-9挂学警港澳]{1}$"
    p_end1 = "^[A-Z0-9]{5}[A-Z0-9挂学警港澳]{1}$"
    box_text_list = [[box, text] for box, text in zip(ocr_result[0].boxes, ocr_result[0].text)]
    for index, box_text in enumerate(box_text_list):
        box, text = box_text
        # 文本长度预先过滤
        if len(text) in [2, 3]:
            match = re.match(p_before, text)
            if match:
                before = match.group()
                if len(before) == 2:
                    before = before[0] + '·' + before[1]
                for box_j, text_j in box_text_list[index + 1:]:
                    # 先用文本长度过滤，然后用上面box的ymax和下面box的ymin差距小于阈值筛选
                    if len(text_j) in [5, 6] and (abs(box_j[1] - box[5]) < DELTA_Y or abs(box_j[3] - box[5]) < DELTA_Y):
                        match_j = re.match(p_end, text_j)
                        if not match_j:
                            match_j = re.match(p_end1, text_j)
                        if match_j:
                            license_plate_list.append(before + match_j.group())

    return license_plate_list


# def predict_ocr(frame, ocr):
#     # OCR检测
#     license_plate_list = []
#     ocr_result = ocr.ocr(frame, cls=False)
#     print(ocr_result)
#     # 赋予空值
#     if ocr_result[0]:
#         for line in ocr_result[0]:
#             txt = line[1][0]
#             # 车牌
#             plate_num = parse_plate_number(txt)
#             if plate_num is not None:
#                 license_plate_list.append(plate_num)
#     return license_plate_list


def get_police(frame_copy, target_infos, police_model):
    """
    识别交警，返回是交警的目标id
    :param frame_copy:
    :param target_infos:
    :param police_model:
    :return:
    """
    police_model_input = dict()
    police_result = []
    for index, info in enumerate(target_infos):
        # track_id, p1, p2, label, conf, num, s, polygon = info
        p1 = info['p_left_up']
        p2 = info['p_right_bottom']
        police_model_input[index] = [p1[0], p1[1], p2[0], p2[1]]
    if police_model_input:
        police_result = predict_police(frame_copy, police_model_input, police_model)
    return police_result


def output_person_targets(left_person, person_clusters, save_dir, output_person_dir):
    index = 1
    paths = []
    for tid in left_person:
        name_index = person_clusters[tid]["max_info"][-1]
        source_path = (
            save_dir
            / (str(tid) + "_person")
            / (str(tid) + "_" + str(name_index) + ".png")
        )
        target_path = output_person_dir / (str(index) + ".png")
        # target_path = Path(PERSON_FACE_SAVE_PATH) / (str(index) + '.png')
        shutil.copyfile(source_path, target_path)
        index += 1
        paths.append(target_path)
    return paths


def output_car_targets(left_cars, save_dir, output_car_dir):
    index = 1
    paths = []
    for licensex in left_cars:
        source_path = save_dir / (licensex + "_car") / "0.png"
        target_path = output_car_dir / (str(index) + ".png")
        # target_path = Path(CAR_FACE_SAVE_PATH / (str(index) + '.png'))
        shutil.copyfile(source_path, target_path)
        index += 1
        paths.append(target_path)
    return paths


def det_articulation(image=None, img_path=None):
    if image is None and not img_path:
        return 0

    if image is None and img_path:
        image = cv2.imread(img_path)

    # 静态计算方法
    result_static = cv2.quality.QualityBRISQUE_compute(
        image, ARTICULATION_MODEL_PATH, ARTICULATION_RANGE_PATH
    )
    score = np.mean([i for i in result_static if (i != 0 and not np.isinf(i))])
    score = 0 if np.isnan(score) else score
    f_score = round(100 - score, 2)
    return f_score


def calculate_iou(box1, box2):
    # box1 和 box2 分别是 [x, y, w, h] 格式的边界框
    x1_min, y1_min, x1_max, y1_max = box1
    x2_min, y2_min, x2_max, y2_max = box2
    w1 = x1_max - x1_min
    h1 = y1_max - y1_min
    w2 = x2_max - x2_min
    h2 = y2_max - y2_min

    # # 计算边界框的四个坐标
    # x1_min, y1_min, x1_max, y1_max = x1, y1, x1 + w1, y1 + h1
    # x2_min, y2_min, x2_max, y2_max = x2, y2, x2 + w2, y2 + h2

    # 计算交集的坐标
    intersection_x_min = max(x1_min, x2_min)
    intersection_y_min = max(y1_min, y2_min)
    intersection_x_max = min(x1_max, x2_max)
    intersection_y_max = min(y1_max, y2_max)

    # 计算交集的面积
    intersection_area = max(0, intersection_x_max - intersection_x_min) * max(
        0, intersection_y_max - intersection_y_min
    )

    # 计算两个边界框的面积
    box1_area = w1 * h1
    box2_area = w2 * h2

    # 计算并比
    iou = intersection_area / (box1_area + box2_area - intersection_area)

    return iou


# 使用yolo预测交警模型
# def predict_police(frame, target_boxes, police_model):
#     """
#     :param frame:
#     :param target_boxes: {tid1: box1, tid2: box2}
#     :param police_model:
#     :return: 返回是交警的目标id列表
#     """

#     # copy_frame = frame.copy()
#     police_reslist = []
#     # 模型加载路径
#     result = police_model(frame)

#     for r in result:
#         boxes = r.boxes
#         for box in boxes:
#             b = box.xyxy[0]
#             c = int(box.cls)
#             confidence = round(float(box.conf), 2)
#             if confidence < 0.5:
#                 continue
#             # p1, p2 = get_coordinate(b)
#             police_bbox = [b[0], b[1], b[2], b[3]]
#             # 遍历人的边界框
#             for key, person_bbox in target_boxes.items():
#                 iou = calculate_iou(police_bbox, person_bbox)
#                 if iou >= POLICE_IOU:  # 根据需要调整IoU阈值
#                     police_reslist.append(key)
#     return police_reslist


# 使用fastdeploy预测交警模型
def predict_police(frame, target_boxes, police_model):
    """
    :param frame:
    :param target_boxes: {tid1: box1, tid2: box2}
    :param police_model:
    :return: 返回是交警的目标id列表
    """

    # copy_frame = frame.copy()
    police_reslist = []
    # 模型加载路径
    result = police_model.predict(frame)
    # 预测结果
    box_list = result.boxes
    score_list = result.scores
    label_list = result.label_ids
    for i, confidence in enumerate(score_list):
        if confidence <= 0.5: # 这个阈值告诉孔凡平让其添加到配置文件
            continue
        police_bbox = box_list[i]
        for key, person_bbox in target_boxes.items():
            iou = calculate_iou(police_bbox, person_bbox)
            if iou >= POLICE_IOU:  # 根据需要调整IoU阈值
                police_reslist.append(key)
    return police_reslist